##############################################################################
#                      Data Expedition 2017                                  #
# Satellite Tagging Records of Deep Divers off Cape Hatteras, North Carolina #
##############################################################################

################################
#### Part I: Clean Tag Data ####
################################

#Make sure you've already installed these packages before starting ('Tools'--> 'Install Packages', once installed, you'll still need to run library function)
library(ggplot2)
library(plyr)

load("alltags_sim.RData")

#Check for too short durations for deep dives (max speed with buffer)
#Assign max speed from literature
#Tyack et al. 2006 says average vertical speed around 1.6m/s
max_speed<-3 #meters per second
max_duration<-120 #min
#Average descent speed ~1.5m/s, max around 2m/s

### 1. Use equation to flag points where dive is too fast or too long ###

for (i in 1:length(alltags_sim)){
  
  ctag<-alltags_sim[[i]]  
  
  ctag$Start<-as.POSIXct(ctag$Start)
  
  ctag$End<-as.POSIXct(ctag$End)
  
  ctag$DurationMin<-as.numeric(ctag$DurationMin)/60 #in minutes
  
  ctag$lv<-(-60*max_speed)*ctag$DurationMin
  
  ctag$check<-ifelse(as.numeric(ctag$DepthMin)*-1<ctag$lv,"FAIL","PASS" )
  
  ctag$check[ctag$DurationMin>=max_duration]<-"FAIL"
  
  ctag$check[ctag$What!="Dive"]<-NA
  
  alltags_sim[[i]]<-ctag
  
}

i=1
dev.new()
par(mfrow=c(2,1))

x<-alltags_sim[[i]]
n<-alltags_sim[[i]][1,1]
x<-x[which(x$What=="Dive"),]

color_scheme<-with(x,data.frame(status=c("FAIL","PASS"),color = c("red", "black"))) #Notice, red = fails test, black = pass

#Plot 1.1 

plot(as.numeric(x$DepthMin)*-1~x$DurationMin,
     col=color_scheme$color[match(x$check,color_scheme$status)],
     ylim=c(-4000,0),main=n, 
     xlab="Duration (min)", ylab="Depth (m)")
abline(a=0, b=(-60*max_speed), lty=2)

#Plot 1.2 Plot against time

plot((as.numeric(x$DepthMin)*-1)~as.POSIXct(x$Start),
     col=color_scheme$color[match(x$check,color_scheme$status)], 
     xlab="Duration (min)", ylab="Depth (m)", main=n)
v=x$Start[which(x$check=="FAIL")][1]
abline(v=v, lty=2)

i=i+1 #will need to highlight and run multiple times, so it will run a new plot each time 

### 2. Censor tags ###

censor_tags<-c("ZcTagsim20","ZcTagsim25")

for (i in 1:length(censor_tags)){
  tag<-get(censor_tags[i], alltags_sim)
  failpoint<-min(which(tag$check=="FAIL"))-1
  tag<-tag[1:failpoint,]
  alltags_sim[[which(names(alltags_sim) %in% censor_tags)[i]]]<-tag
}

alltags_clean <- alltags_sim

save(alltags_clean, file="alltags_clean.RData")

rm(list=ls())


############################################
#### Part II: Exploring Individual Tags ####
############################################

### 3. Plotting Dive Depth for GmTagsim1 ###

#First, we'll prepare the data
load('alltags_clean.RData')

GmTagsim1 <- alltags_clean[[1]]  #makes into normal data frame that you can work with

GmTagsim1$DepthMin<- as.numeric(GmTagsim1$DepthMin) #changes data in this column from character to numeric (change occurs in GmTagsim1 data frame, not in the larger list)

GmTagsim1 <- subset(GmTagsim1, GmTagsim1$What == "Dive" | GmTagsim1$What== "Surface" | GmTagsim1$What == "GAP") #Pulls out messages, leaving just Dive and Surface data (and gaps)

GmTagsim1$DepthMin[is.na(GmTagsim1$DepthMin)] <- 0 #Turns all surface data with no depth to 0

GmTagsim1$GmDepth = -1 * GmTagsim1$DepthMin #Makes depth data negative so it's intuitive when graphed

##Plot 3.1 
#Date vs. dive depth for GmTagsim1
dev.new() #this command tells R to open your next plot in a new window, you'll need to highlight this line and the next 6 lines, then hit 'command' + 'enter'
ggplot() +
  geom_line(data = GmTagsim1, size =0.25, aes(x=Start, y=GmDepth)) +
  geom_point(data = GmTagsim1, alpha =0.6, aes(x= Start, y = GmDepth, colour=GmDepth)) +
  ggtitle ("GmTagsim1") +
  xlab ("Date") +
  ylab ("Depth (m)") +
  labs (color = "Duration (min)")


##Plot 3.2 
#Histogram of dive depth distribution
dev.new() 
ggplot(GmTagsim1, aes(GmTagsim1$DepthMin)) +
  geom_histogram(fill = "skyblue2", color = "skyblue4", binwidth  = 40) +
  scale_y_sqrt() +
  ggtitle ("GmTagsim1 Dive Depth Distribution") +
  xlab ("Dive Depth(m)") +
  ylab ("Count")


### 4. Plotting Dive Duration for GmTagsim1 ###

GmTagsim1 <- subset(GmTagsim1, GmTagsim1$What == "Dive")
GmTagsim1$DurationMin<- as.numeric(GmTagsim1$DurationMin) #changes to numeric. Remember, we already changed this column to minutes when cleaning the data. The DurationMax column is still in seconds

##4.1 Histogram of Dive Duration
dev.new()
ggplot(GmTagsim1, aes(GmTagsim1$DurationMin)) +
  geom_histogram(fill = "coral2", color = "coral4", binwidth=1) +
  scale_y_sqrt()+
  ggtitle ("GmTagsim1 Dive Duration Distribution") +
  xlab ("Dive Duration(min)") +
  ylab ("Count")


### 5. Plotting Dive Depth versus Dive Duration for GmTagsim1 ###

##Plot 5.1
dev.new() 
ggplot(data = GmTagsim1, aes(x=GmTagsim1$DepthMin, y=GmTagsim1$DurationMin)) +
  geom_point(aes(color=GmTagsim1$DeployID)) +
  ggtitle ("GmTagsim1 Depth Duration Distribution") +
  xlab ("Dive Depth(m)") +
  ylab ("Dive Duration(min)") +
  ylim (0, 30) +
  labs (color = "Tag ID")

### 6. Repeat Steps 3-5 for a beaked whale tag ###

##Hint-- copy and paste R script from steps 1-3 and change code to reflect beaked whale data (everywhere it says 'GmTag' should change to 'ZcTag')



##########################################################################
#### Part III: Visualizing Individual Variation and Comparing Species ####
##########################################################################

### To do this, we'll graph multiple tags together

#First, separate pilot whale and beaked whale tags
GmTaglist <- alltags_clean[grep("GmTag", alltags_clean)]
ZcTaglist <- alltags_clean[grep("ZcTag", alltags_clean)]

#Then turn these lists into single data frames
GmTags <- ldply (GmTaglist) #moves list into single data frame
ZcTags <- ldply (ZcTaglist)


### 7. Plotting Dive Depth and Duration distribution for all pilot whales ###

#Make sure all the columns you plan to work with are considered numeric data
GmTags$DepthMin<- as.numeric(GmTags$DepthMin)
GmTags$DurationMin<- as.numeric(GmTags$DurationMin)

#Plot 7.1 Pilot Whale Point Plot

dev.new()
ggplot(data = GmTags, aes(x=GmTags$DepthMin, y=GmTags$DurationMin)) +
  geom_point(aes(color=GmTags$DeployID, alpha = 0.01)) +
  ggtitle ("Pilot Whale Depth Duration Distribution") +
  xlab ("Dive Depth(m)") +
  ylab ("Dive Duration(min)") +
  ylim (0, 30) +
  labs (color = "Tag ID")

#It's difficult to see everything on this plot. Let's try visualizing it a different way. The following code will plot a distribution curve for dive duration for each individual. 
GmDives <- subset(GmTags, GmTags$What == "Dive") #first make sure you're only looking at the dive durations for actual dives

#Plot 7.2 Pilot Whale Density Plot
#If you think dive depth might be more representative than duration, you could easily alter this plot by changing 'GmDives$DurationMin' to 'GmDives$DepthMin', and changing the xlim to (0,1300). You'd also want to change the title and labels.
dev.new()
ggplot(GmDives, aes(GmDives$DurationMin)) +
  geom_density(aes(colour=GmDives$DeployID, alpha =0.1)) +
  ggtitle ("Pilot Whale Dive Duration Variation") +
  xlim(0,160) +
  xlab ("Dive Duration(min)") +
  ylab ("Count") +
  labs (color = "Tag ID")


### 8. Now repeat Step 7 for beaked whales ###

#Make sure all the columns you plan to work with are considered numeric data

#Plot 8.1 Beaked Whale Point Plot(you can try a different xlimit here)

#Separate dives out

#Plot 8.2 Beaked Whale Density plot

##############################
#### Part IV: Tag Summary ####
##############################

### 9. Create and save a data frame that summarizes data for each tag ###

tagsummary<-list()

for (i in 1:length(alltags_clean)) {
  
  current<-alltags_clean[[i]]
  start_date<-as.character(current$Start[[1]])
  tagduration<-max(current$End)-min(current$Start)
  numdives<-length(current$What[current$What=="Dive"])
  maxdivedepth<-max(na.omit(as.numeric(current$DepthMin)))
  maxdiveduration<-max(na.omit(as.numeric(current$DurationMin[current$What=="Dive"])))/60
  results<-c(current[1,1], start_date, tagduration, numdives, maxdivedepth, maxdiveduration)
  tagsummary[[i]]<-results
}

alltagsummary<-as.data.frame(do.call("rbind", tagsummary))

names(alltagsummary)<-c("tagID", "start_date", "tagDuration", "numdives", "maxdivedepth", "maxdiveduration")

View(alltagsummary)

write.csv(alltagsummary, "tag_means.csv")

###############################################################################################################
# THE END #
# Please contact Vivienne Foroughirad (vivienne.foroughirad@duke.edu) or Jillian Wisse (jillian.wisse@duke.edu) with any questions pertaining to this Data Expedition activity 
###############################################################################################################
